@@ -61,6 +61,8 @@
 #include "totemComplexPlugin.h"
 #elif defined(TOTEM_NARROWSPACE_PLUGIN)
 #include "totemNarrowSpacePlugin.h"
+#elif defined(TOTEM_MULLY_PLUGIN)
+#include "totemMullYPlugin.h"
 #else
 #error Unknown plugin type
 #endif
@@ -125,7 +127,44 @@ cb_stop_sending_data (DBusGProxy  *proxy,
 		return;
 	}
 
-	plugin->stream = NULL;
+	plugin->stream = nsnull;
+}
+
+static char *
+get_real_mimetype (const char *mimetype)
+{
+	const totemPluginMimeEntry *mimetypes;
+	PRUint32 count;
+
+	totemScriptablePlugin::PluginMimeTypes (&mimetypes, &count);
+
+	for (PRUint32 i = 0; i < count; ++i) {
+		if (strcmp (mimetypes[i].mimetype, mimetype) == 0) {
+			if (mimetypes[i].mime_alias != NULL)
+				return g_strdup (mimetypes[i].mime_alias);
+			else
+				return g_strdup (mimetype);
+		}
+	}
+
+	g_warning ("Real mime-type for '%s' not found\n", mimetype);
+	return NULL;
+}
+
+static gboolean
+is_mimetype_supported (const char *mimetype)
+{
+	const totemPluginMimeEntry *mimetypes;
+	PRUint32 count;
+
+	totemScriptablePlugin::PluginMimeTypes (&mimetypes, &count);
+
+	for (PRUint32 i = 0; i < count; ++i) {
+		if (strcmp (mimetypes[i].mimetype, mimetype) == 0)
+			return TRUE;
+	}
+
+	return FALSE;
 }
 
 static gboolean
@@ -136,6 +175,8 @@ is_supported_scheme (const char *url)
 
 	if (g_str_has_prefix (url, "mms:") != FALSE)
 		return FALSE;
+	if (g_str_has_prefix (url, "rtsp:") != FALSE)
+		return FALSE;
 
 	return TRUE;
 }
@@ -149,6 +190,7 @@ totem_plugin_fork (totemPlugin *plugin)
 	char **argv;
 	GError *err = NULL;
 	totemScriptablePlugin *scriptable = plugin->scriptable;
+	gboolean use_fd = FALSE;
 
 	/* Make sure we don't get both an XID and hidden */
 	if (plugin->window && plugin->hidden) {
@@ -227,23 +269,19 @@ totem_plugin_fork (totemPlugin *plugin)
  		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_PLAYLIST));
  		g_ptr_array_add (arr, g_strdup (plugin->local));
  	} else {
-		if (plugin->local == NULL) {
-			if (is_supported_scheme (plugin->local) == FALSE)
-				g_ptr_array_add (arr, g_strdup (plugin->src));
-			else
-				g_ptr_array_add (arr, g_strdup ("fd://0"));
+		/* plugin->local is only TRUE for playlists */
+		if (plugin->is_supported_src == FALSE) {
+			g_ptr_array_add (arr, g_strdup (plugin->src));
 		} else {
-			if (is_supported_scheme (plugin->src) == FALSE)
-				g_ptr_array_add (arr, g_strdup (plugin->src));
-			else
-				g_ptr_array_add (arr, g_strdup ("fd://0"));
+			g_ptr_array_add (arr, g_strdup ("fd://0"));
+			use_fd = TRUE;
 		}
- 	}
+	}
 
 	g_ptr_array_add (arr, NULL);
 	argv = (char **) g_ptr_array_free (arr, FALSE);
 
-#ifdef TOTEM_DEBUG
+#ifdef GNOME_ENABLE_DEBUG
 	{
 		GString *s;
 		int i;
@@ -260,9 +298,8 @@ totem_plugin_fork (totemPlugin *plugin)
 #endif
 
 	if (g_spawn_async_with_pipes (NULL, argv, NULL,
-				G_SPAWN_DO_NOT_REAP_CHILD,
-				NULL, NULL, &plugin->player_pid,
-				&plugin->send_fd, NULL, NULL, &err) == FALSE)
+				(GSpawnFlags) 0, NULL, NULL, &plugin->player_pid,
+				use_fd ? &plugin->send_fd : NULL, NULL, NULL, &err) == FALSE)
 	{
 		D("Spawn failed");
 
@@ -277,7 +314,8 @@ totem_plugin_fork (totemPlugin *plugin)
 
 	g_strfreev (argv);
 
-	fcntl(plugin->send_fd, F_SETFL, O_NONBLOCK);
+	if (plugin->send_fd > 0)
+		fcntl(plugin->send_fd, F_SETFL, O_NONBLOCK);
 
 	/* now wait until startup is complete */
 	plugin->got_svc = FALSE;
@@ -315,7 +353,7 @@ totem_plugin_fork (totemPlugin *plugin)
 
 		if (plugin->player_pid) {
 			kill (plugin->player_pid, SIGKILL);
-			waitpid (plugin->player_pid, NULL, 0);
+			g_spawn_close_pid (plugin->player_pid);
 			plugin->player_pid = 0;
 		}
 		return FALSE;
@@ -332,7 +370,6 @@ totem_plugin_fork (totemPlugin *plugin)
 	dbus_g_proxy_connect_signal (plugin->proxy, "StopSendingData",
 				     G_CALLBACK (cb_stop_sending_data),
 				     plugin, NULL);
-	g_message ("%p plugin", plugin);
 
 	g_free (plugin->wait_for_svc);
 	D("Done forking, new proxy=%p", plugin->proxy);
@@ -367,6 +404,7 @@ totem_plugin_new_instance (NPMIMEType mimetype,
 {
 	totemPlugin *plugin;
 	GError *e = NULL;
+	gboolean need_req = FALSE;
 	int i;
 
 	D("totem_plugin_new_instance");
@@ -446,64 +484,58 @@ totem_plugin_new_instance (NPMIMEType mimetype,
 	if (strcmp (mimetype, "video/quicktime") != 0) {
 		plugin->cache = TRUE;
 	}
-	plugin->mimetype = g_strdup (mimetype);
+	/* Find the "real" mime-type */
+	plugin->mimetype = get_real_mimetype (mimetype);
 
-	for (i=0; i<argc; i++) {
+	for (i = 0; i < argc; i++) {
 		printf ("argv[%d] %s %s\n", i, argn[i], argv[i]);
 		if (g_ascii_strcasecmp (argn[i],"width") == 0) {
 			plugin->width = strtol (argv[i], NULL, 0);
-		}
-		if (g_ascii_strcasecmp (argn[i], "height") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "height") == 0) {
 			plugin->height = strtol (argv[i], NULL, 0);
-		}
-		if (g_ascii_strcasecmp (argn[i], "src") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "src") == 0) {
 			plugin->src = resolve_relative_uri (docURI, argv[i]);
-			//plugin->srcSupported = is_supported_scheme (plugin->src);
-		}
-		/* Windows Media Player parameter */
-		if (g_ascii_strcasecmp (argn[i], "filename") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "filename") == 0) {
+			/* Windows Media Player parameter */
 			if (plugin->src == NULL) {
 				plugin->src = resolve_relative_uri (docURI, argv[i]);
-				g_message ("plugin->src = %s", plugin->src);
+				need_req = TRUE;
 			}
-		}
-		if (g_ascii_strcasecmp (argn[i], "href") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "href") == 0) {
 			plugin->href = resolve_relative_uri (docURI, argv[i]);
-		}
-		if (g_ascii_strcasecmp (argn[i], "cache") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "cache") == 0) {
 			plugin->cache = TRUE;
 			if (g_ascii_strcasecmp (argv[i], "false") == 0) {
 				plugin->cache = FALSE;
 			}
-		}
-		if (g_ascii_strcasecmp (argn[i], "target") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "target") == 0) {
 			plugin->target = g_strdup (argv[i]);
-		}
-		/* Quicktime parameter */
-		if (g_ascii_strcasecmp (argn[i], "controller") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "controller") == 0) {
+			/* Quicktime parameter */
 			if (g_ascii_strcasecmp (argv[i], "false") == 0) {
 				plugin->controller_hidden = TRUE;
 			}
 			//FIXME see http://www.htmlcodetutorial.com/embeddedobjects/_EMBED_CONTROLS.html
-		}
-		/* Windows Media Player parameter */
-		if (g_ascii_strcasecmp (argn[i], "uimode") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "uimode") == 0) {
+			/* Windows Media Player parameter */
 			if (g_ascii_strcasecmp (argv[i], "none") == 0) {
 				plugin->controller_hidden = TRUE;
 			}
-		}
-		if (g_ascii_strcasecmp (argn[i], "hidden") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "showcontrols") == 0) {
+			if (g_ascii_strcasecmp (argv[i], "false") == 0) {
+				plugin->controller_hidden = TRUE;
+			}
+		} else if (g_ascii_strcasecmp (argn[i], "hidden") == 0) {
 			if (g_ascii_strcasecmp (argv[i], "false") != 0) {
 				plugin->hidden = TRUE;
 			}
-		}
-		if (g_ascii_strcasecmp (argn[i], "autostart") == 0
+		} else if (g_ascii_strcasecmp (argn[i], "autostart") == 0
 				|| g_ascii_strcasecmp (argn[i], "autoplay") == 0) {
-			if (g_ascii_strcasecmp (argv[i], "false") == 0) {
+			if (g_ascii_strcasecmp (argv[i], "false") == 0
+					|| g_ascii_strcasecmp (argv[i], "0") == 0) {
 				plugin->noautostart = TRUE;
 			}
-		}
-		if (g_ascii_strcasecmp (argn[i], "loop") == 0
+		} else if (g_ascii_strcasecmp (argn[i], "loop") == 0
 				|| g_ascii_strcasecmp (argn[i], "repeat") == 0) {
 			if (g_ascii_strcasecmp (argv[i], "true") == 0) {
 				plugin->repeat = TRUE;
@@ -511,16 +543,24 @@ totem_plugin_new_instance (NPMIMEType mimetype,
 
 			// FIXME Doesn't handle playcount, or loop with numbers
 			// http://www.htmlcodetutorial.com/embeddedobjects/_EMBED_LOOP.html
-		}
-		if (g_ascii_strcasecmp (argn[i], "starttime") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "starttime") == 0) {
 			//FIXME see http://www.htmlcodetutorial.com/embeddedobjects/_EMBED_STARTTIME.html
-		}
-		if (g_ascii_strcasecmp (argn[i], "endtime") == 0) {
+		} else if (g_ascii_strcasecmp (argn[i], "endtime") == 0) {
 			//FIXME see above
 		}
 	}
 
-	NS_IF_RELEASE (docURI);	
+	NS_IF_RELEASE (docURI);
+
+	plugin->is_supported_src = is_supported_scheme (plugin->src);
+
+	/* If filename is used, we need to request the stream ourselves */
+	if (need_req == TRUE) {
+		if (plugin->is_supported_src != FALSE) {
+			CallNPN_GetURLProc(mozilla_functions.geturl,
+					instance, plugin->src, NULL);
+		}
+	}
 
 	return NPERR_NO_ERROR;
 }
@@ -543,12 +583,15 @@ totem_plugin_destroy_instance (NPP instance,
 
 	plugin->scriptable->UnsetPlugin ();
 
-	if (plugin->send_fd >= 0)
-		close(plugin->send_fd);
+	if (plugin->send_fd >= 0) {
+		close (plugin->send_fd);
+		plugin->send_fd = -1;
+	}
 
 	if (plugin->player_pid) {
 		kill (plugin->player_pid, SIGKILL);
-		waitpid (plugin->player_pid, NULL, 0);
+		g_spawn_close_pid (plugin->player_pid);
+		plugin->player_pid = 0;
 	}
 
 	NS_RELEASE (plugin->scriptable);
@@ -590,9 +633,15 @@ totem_plugin_set_window (NPP instance,
 			printf ("ack.  window changed!\n");
 		}
 	} else {
-		/* If not, wait for the first bits of data to come */
-		D("waiting for data to come");
+		/* If not, wait for the first bits of data to come,
+		 * but not when the stream type isn't supported */
 		plugin->window = (Window) window->window;
+		if (plugin->stream && plugin->is_supported_src) {
+			if (!totem_plugin_fork (plugin))
+				return NPERR_GENERIC_ERROR;
+		} else {
+			D("waiting for data to come");
+		}
 	}
 
 	D("leaving plugin_set_window");
@@ -620,6 +669,9 @@ totem_plugin_new_stream (NPP instance,
 
 	D("plugin_new_stream type: %s url: %s", type, plugin->src);
 
+	if (is_mimetype_supported (type) == FALSE)
+		return NPERR_INVALID_PLUGIN_ERROR;
+
 	//FIXME need to find better semantics?
 	//what about saving the state, do we get confused?
 	if (g_str_has_prefix (plugin->src, "file://")) {
@@ -640,7 +692,7 @@ totem_plugin_destroy_stream (NPP instance,
 			     NPStream* stream,
 			     NPError reason)
 {
-	D("plugin_destroy_stream");
+	D("plugin_destroy_stream, reason: %d", reason);
 
 	if (instance == NULL) {
 		D("totem_plugin_destroy_stream instance is NULL");
@@ -649,8 +701,10 @@ totem_plugin_destroy_stream (NPP instance,
 
 	totemPlugin *plugin = (totemPlugin *) instance->pdata;
 
-	close(plugin->send_fd);
-	plugin->send_fd = -1;
+	if (plugin->send_fd >= 0) {
+		close(plugin->send_fd);
+		plugin->send_fd = -1;
+	}
 
 	plugin->stream = nsnull;
 
@@ -669,8 +723,7 @@ static int32 totem_plugin_write_ready (NPP instance, NPStream *stream)
 
 	plugin = (totemPlugin *) instance->pdata;
 
-//	if (!plugin->srcSupported)
-	if (is_supported_scheme (plugin->src) == FALSE)
+	if (plugin->is_supported_src == FALSE)
 		return 0;
 
 	if (plugin->send_fd < 0)
@@ -678,7 +731,7 @@ static int32 totem_plugin_write_ready (NPP instance, NPStream *stream)
 
 	fds.events = POLLOUT;
 	fds.fd = plugin->send_fd;
-	if (plugin->send_fd > 0 && poll (&fds, 1, 0) > 0)
+	if (poll (&fds, 1, 0) > 0)
 		return (PLUGIN_STREAM_CHUNK_SIZE);
 
 	return 0;
@@ -699,15 +752,16 @@ static int32 totem_plugin_write (NPP instance, NPStream *stream, int32 offset,
 		return -1;
 
 	if (!plugin->player_pid) {
-		/* FIXME this looks wrong since it'll look at the current data buffer,
-		 * not the cumulative data since the stream started 
-		 */
-		if (!plugin->stream)
-		{
-			g_message ("No stream in NPP_Write!?");
+		if (!plugin->stream) {
+			g_warning ("No stream in NPP_Write!?");
 			return -1;
 		}
 
+		plugin->tried_write = TRUE;
+
+		/* FIXME this looks wrong since it'll look at the current data buffer,
+		 * not the cumulative data since the stream started 
+		 */
 		if (totem_pl_parser_can_parse_from_data ((const char *) buffer, len, TRUE /* FIXME */) != FALSE) {
 			D("Need to wait for the file to be downloaded completely");
 			plugin->is_playlist = TRUE;
@@ -721,8 +775,7 @@ static int32 totem_plugin_write (NPP instance, NPStream *stream, int32 offset,
 	if (plugin->send_fd < 0)
 		return -1;
 
-	// if (!plugin->srcSupported)
-	if (is_supported_scheme (plugin->src) == FALSE)
+	if (plugin->is_supported_src == FALSE)
 		return -1;
 
 	ret = write (plugin->send_fd, buffer, len);
@@ -760,6 +813,11 @@ static void totem_plugin_stream_as_file (NPP instance, NPStream *stream,
 	if (plugin == NULL)
 		return;
 
+	if (plugin->tried_write == FALSE) {
+		plugin->is_playlist = totem_pl_parser_can_parse_from_filename
+			(fname, TRUE);
+	}
+
 	if (!plugin->player_pid && plugin->is_playlist != FALSE) {
 		plugin->local = g_filename_to_uri (fname, NULL, NULL);
 		totem_plugin_fork (plugin);
@@ -769,6 +827,9 @@ static void totem_plugin_stream_as_file (NPP instance, NPStream *stream,
 			return;
 	}
 
+	if (plugin->is_playlist != FALSE)
+		return;
+
 	if (!dbus_g_proxy_call (plugin->proxy, "SetLocalFile", &err,
 			G_TYPE_STRING, fname, G_TYPE_INVALID,
 			G_TYPE_INVALID)) {
